@@ -32,8 +32,9 @@ struct Shishi_ap
   Shishi_asn1 encapreppart;
   int authenticatorcksumkeyusage;
   int authenticatorkeyusage;
+  int authenticatorcksumtype;
   char *authenticatorcksumdata;
-  int authenticatorcksumdatalen;
+  size_t authenticatorcksumdatalen;
 };
 
 /**
@@ -54,7 +55,7 @@ shishi_ap (Shishi * handle, Shishi_ap ** ap)
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "Could not create Authenticator: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return res;
     }
 
@@ -83,13 +84,11 @@ shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
 {
   Shishi_ap *lap;
 
-  *ap = malloc (sizeof (**ap));
-  if (*ap == NULL)
-    return SHISHI_MALLOC_ERROR;
+  *ap = xcalloc (1, sizeof (**ap));
   lap = *ap;
-  memset (lap, 0, sizeof (*lap));
 
   lap->handle = handle;
+  lap->authenticatorcksumtype = SHISHI_NO_CKSUMTYPE;
   lap->authenticatorcksumkeyusage = SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR_CKSUM;
   lap->authenticatorkeyusage = SHISHI_KEYUSAGE_APREQ_AUTHENTICATOR;
 
@@ -97,7 +96,7 @@ shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
   if (lap->authenticator == NULL)
     {
       shishi_error_printf (handle, "Could not create Authenticator: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -105,7 +104,7 @@ shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
   if (lap->apreq == NULL)
     {
       shishi_error_printf (handle, "Could not create AP-REQ: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -113,7 +112,7 @@ shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
   if (lap->aprep == NULL)
     {
       shishi_error_printf (handle, "Could not create AP-REP: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
@@ -121,13 +120,31 @@ shishi_ap_nosubkey (Shishi * handle, Shishi_ap ** ap)
   if (lap->encapreppart == NULL)
     {
       shishi_error_printf (handle, "Could not create EncAPRepPart: %s\n",
-			   shishi_strerror_details (handle));
+			   shishi_error (handle));
       return SHISHI_ASN1_ERROR;
     }
 
   return SHISHI_OK;
 }
 
+/**
+ * shishi_ap_done:
+ * @ap: structure that holds information about AP exchange
+ *
+ * Deallocate resources associated with AP exchange.  This should be
+ * called by the application when it no longer need to utilize the AP
+ * exchange handle.
+ **/
+void
+shishi_ap_done (Shishi_ap * ap)
+{
+  shishi_asn1_done (ap->handle, ap->authenticator);
+  shishi_asn1_done (ap->handle, ap->apreq);
+  shishi_asn1_done (ap->handle, ap->aprep);
+  shishi_asn1_done (ap->handle, ap->encapreppart);
+  free (ap);
+}
+
 /**
  * shishi_ap_set_tktoptions:
  * @ap: structure that holds information about AP exchange
@@ -173,7 +190,7 @@ shishi_ap_set_tktoptions (Shishi_ap * ap, Shishi_tkt * tkt, int options)
 int
 shishi_ap_set_tktoptionsdata (Shishi_ap * ap,
 			      Shishi_tkt * tkt,
-			      int options, char *data, int len)
+			      int options, const char *data, size_t len)
 {
   int rc;
 
@@ -192,15 +209,18 @@ shishi_ap_set_tktoptionsdata (Shishi_ap * ap,
 }
 
 /**
- * shishi_ap_set_tktoptionsasn1:
+ * shishi_ap_set_tktoptionsasn1usage:
  * @ap: structure that holds information about AP exchange
  * @tkt: ticket to set in AP.
  * @options: AP-REQ options to set in AP.
  * @node: input ASN.1 structure to store as authenticator checksum data.
+ * @field: field in ASN.1 structure to use.
+ * @authenticatorcksumkeyusage: key usage for checksum in authenticator.
+ * @authenticatorkeyusage: key usage for authenticator.
  *
  * Set ticket, options and authenticator checksum data using
  * shishi_ap_set_tktoptionsdata().  The authenticator checksum data is
- * the DER encoding of the ASN.1 structure provided.
+ * the DER encoding of the ASN.1 field provided.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
@@ -282,7 +302,8 @@ shishi_ap_tktoptions (Shishi * handle,
 int
 shishi_ap_tktoptionsdata (Shishi * handle,
 			  Shishi_ap ** ap,
-			  Shishi_tkt * tkt, int options, char *data, int len)
+			  Shishi_tkt * tkt, int options,
+			  const char *data, size_t len)
 {
   int rc;
 
@@ -298,12 +319,15 @@ shishi_ap_tktoptionsdata (Shishi * handle,
 }
 
 /**
- * shishi_ap_tktoptionsasn1:
+ * shishi_ap_tktoptionsasn1usage:
  * @handle: shishi handle as allocated by shishi_init().
  * @ap: pointer to new structure that holds information about AP exchange
  * @tkt: ticket to set in newly created AP.
  * @options: AP-REQ options to set in newly created AP.
  * @node: input ASN.1 structure to store as authenticator checksum data.
+ * @field: field in ASN.1 structure to use.
+ * @authenticatorcksumkeyusage: key usage for checksum in authenticator.
+ * @authenticatorkeyusage: key usage for authenticator.
  *
  * Create a new AP exchange using shishi_ap(), and set ticket, options
  * and authenticator checksum data from the DER encoding of the ASN.1
@@ -375,7 +399,7 @@ shishi_ap_tkt_set (Shishi_ap * ap, Shishi_tkt * tkt)
  * SHISHI_TOO_SMALL_BUFFER if buffer provided was too small.
  **/
 int
-shishi_ap_authenticator_cksumdata (Shishi_ap * ap, char *out, int *len)
+shishi_ap_authenticator_cksumdata (Shishi_ap * ap, char *out, size_t * len)
 {
   if (*len < ap->authenticatorcksumdatalen)
     return SHISHI_TOO_SMALL_BUFFER;
@@ -397,13 +421,40 @@ shishi_ap_authenticator_cksumdata (Shishi_ap * ap, char *out, int *len)
  **/
 void
 shishi_ap_authenticator_cksumdata_set (Shishi_ap * ap,
-				       char *authenticatorcksumdata,
-				       int authenticatorcksumdatalen)
+				       const char *authenticatorcksumdata,
+				       size_t authenticatorcksumdatalen)
 {
   ap->authenticatorcksumdata = authenticatorcksumdata;
   ap->authenticatorcksumdatalen = authenticatorcksumdatalen;
 }
 
+/**
+ * shishi_ap_authenticatorcksumtype:
+ * @ap: structure that holds information about AP exchange
+ *
+ * Get the Authenticator Checksum Type in the AP exchange.
+ *
+ * Return value: Return the authenticator checksum type.
+ **/
+int
+shishi_ap_authenticator_cksumtype (Shishi_ap * ap)
+{
+  return ap->authenticatorcksumtype;
+}
+
+/**
+ * shishi_ap_authenticator_cksumtype_set:
+ * @ap: structure that holds information about AP exchange
+ * @cksumtype: authenticator checksum type to set in AP.
+ *
+ * Set the Authenticator Checksum Type in the AP exchange.
+ **/
+void
+shishi_ap_authenticator_cksumtype_set (Shishi_ap * ap, int cksumtype)
+{
+  ap->authenticatorcksumtype = cksumtype;
+}
+
 /**
  * shishi_ap_authenticator:
  * @ap: structure that holds information about AP exchange
@@ -411,6 +462,7 @@ shishi_ap_authenticator_cksumdata_set (Shishi_ap * ap,
  * Return value: Returns the Authenticator from the AP exchange, or
  *               NULL if not yet set or an error occured.
  **/
+
 Shishi_asn1
 shishi_ap_authenticator (Shishi_ap * ap)
 {
@@ -474,7 +526,7 @@ shishi_ap_req_set (Shishi_ap * ap, Shishi_asn1 apreq)
  * Return value: Returns SHISHI_OK iff successful.
  **/
 int
-shishi_ap_req_der (Shishi_ap * ap, char **out, size_t *outlen)
+shishi_ap_req_der (Shishi_ap * ap, char **out, size_t * outlen)
 {
   int rc;
 
@@ -524,6 +576,7 @@ int
 shishi_ap_req_build (Shishi_ap * ap)
 {
   int res;
+  int cksumtype;
 
   if (VERBOSE (ap->handle))
     printf ("Building AP-REQ...\n");
@@ -533,20 +586,29 @@ shishi_ap_req_build (Shishi_ap * ap)
   if (res != SHISHI_OK)
     {
       shishi_error_printf (ap->handle, "Could not set ticket in AP-REQ: %s\n",
-			   shishi_strerror_details (ap->handle));
+			   shishi_error (ap->handle));
       return res;
     }
 
-  res = shishi_authenticator_add_cksum (ap->handle, ap->authenticator,
-					shishi_tkt_key (ap->tkt),
-					ap->authenticatorcksumkeyusage,
-					ap->authenticatorcksumdata,
-					ap->authenticatorcksumdatalen);
+  cksumtype = shishi_ap_authenticator_cksumtype (ap);
+  if (cksumtype == SHISHI_NO_CKSUMTYPE)
+    res = shishi_authenticator_add_cksum (ap->handle, ap->authenticator,
+					  shishi_tkt_key (ap->tkt),
+					  ap->authenticatorcksumkeyusage,
+					  ap->authenticatorcksumdata,
+					  ap->authenticatorcksumdatalen);
+  else
+    res = shishi_authenticator_add_cksum_type (ap->handle, ap->authenticator,
+					       shishi_tkt_key (ap->tkt),
+					       ap->authenticatorcksumkeyusage,
+					       cksumtype,
+					       ap->authenticatorcksumdata,
+					       ap->authenticatorcksumdatalen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (ap->handle,
 			   "Could not add checksum to authenticator: %s\n",
-			   shishi_strerror_details (ap->handle));
+			   shishi_error (ap->handle));
       return res;
     }
 
@@ -563,7 +625,7 @@ shishi_ap_req_build (Shishi_ap * ap)
   if (res != SHISHI_OK)
     {
       shishi_error_printf (ap->handle, "Could not set authenticator: %s\n",
-			   shishi_strerror_details (ap->handle));
+			   shishi_error (ap->handle));
       return res;
     }
 
@@ -590,8 +652,7 @@ shishi_ap_req_build (Shishi_ap * ap)
  **/
 int
 shishi_ap_req_process_keyusage (Shishi_ap * ap,
-				Shishi_key * key,
-				int32_t keyusage)
+				Shishi_key * key, int32_t keyusage)
 {
   Shishi_asn1 ticket, authenticator;
   Shishi_tkt *tkt;
@@ -645,6 +706,8 @@ shishi_ap_req_process_keyusage (Shishi_ap * ap,
       return rc;
     }
 
+  /* XXX? verify checksum in authenticator. */
+
   if (VERBOSEASN1 (ap->handle))
     shishi_authenticator_print (ap->handle, stdout, authenticator);
 
@@ -754,7 +817,7 @@ shishi_ap_rep_set (Shishi_ap * ap, Shishi_asn1 aprep)
  * @out: output array with newly allocated DER encoding of AP-REP.
  * @outlen: length of output array with DER encoding of AP-REP.
  *
- * Build AP-REQ using shishi_ap_rep_build() and DER encode it.  @out
+ * Build AP-REP using shishi_ap_rep_build() and DER encode it.  @out
  * is allocated by this function, and it is the responsibility of
  * caller to deallocate it.
  *
@@ -807,7 +870,7 @@ shishi_ap_rep_der_set (Shishi_ap * ap, char *der, size_t derlen)
  * @ap: structure that holds information about AP exchange
  *
  * Checksum data in authenticator and add ticket and authenticator to
- * AP-REQ.
+ * AP-REP.
  *
  * Return value: Returns SHISHI_OK iff successful.
  **/
@@ -987,8 +1050,19 @@ shishi_ap_encapreppart_set (Shishi_ap * ap, Shishi_asn1 encapreppart)
 #define APOPTION_MUTUAL_REQUIRED "mutual-required"
 #define APOPTION_UNKNOWN "unknown"
 
+/**
+ * shishi_ap_option2string:
+ * @option: enumerated AP-Option type, see Shishi_apoptions.
+ *
+ * Convert AP-Option type to AP-Option name string.  Note that @option
+ * must be just one of the AP-Option types, it cannot be an binary
+ * ORed indicating several AP-Options.
+ *
+ * Return value: Returns static string with name of AP-Option that
+ *   must not be deallocated, or "unknown" if AP-Option was not understood.
+ **/
 const char *
-shishi_ap_option2string (int option)
+shishi_ap_option2string (Shishi_apoptions option)
 {
   char *str;
 
@@ -1014,7 +1088,17 @@ shishi_ap_option2string (int option)
   return str;
 }
 
-int
+/**
+ * shishi_ap_string2option:
+ * @str: zero terminated character array with name of AP-Option,
+ *   e.g. "use-session-key".
+ *
+ * Convert AP-Option name to AP-Option type.
+ *
+ * Return value: Returns enumerated type member corresponding to AP-Option,
+ *   or 0 if string was not understood.
+ **/
+Shishi_apoptions
 shishi_ap_string2option (const char *str)
 {
   int option;
