@@ -31,10 +31,10 @@ raw_des_checksum0 (Shishi * handle, int algo,
   size_t tmplen;
   char *p;
   int blen = 8;
-  int hlen = 16;
+  int hlen = (algo == SHISHI_DES_CBC_CRC) ? 4 : 16;
   int rc;
 
-  rc = shishi_randomize (handle, out, blen);
+  rc = shishi_randomize (handle, 0, out, blen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -46,6 +46,10 @@ raw_des_checksum0 (Shishi * handle, int algo,
 
   switch (algo)
     {
+    case SHISHI_DES_CBC_CRC:
+      rc = shishi_crc (handle, tmp, tmplen, &p);
+      break;
+
     case SHISHI_DES_CBC_MD4:
       rc = shishi_md4 (handle, tmp, tmplen, &p);
       break;
@@ -75,10 +79,10 @@ raw_des_checksum1 (Shishi * handle, int algo,
   size_t tmplen;
   char *p;
   int blen = 8;
-  int hlen = 16;
+  int hlen = (algo == SHISHI_DES_CBC_CRC) ? 4 : 16;
   int rc;
 
-  rc = shishi_randomize (handle, out, blen);
+  rc = shishi_randomize (handle, 0, out, blen);
   if (rc != SHISHI_OK)
     return rc;
 
@@ -92,6 +96,10 @@ raw_des_checksum1 (Shishi * handle, int algo,
 
   switch (algo)
     {
+    case SHISHI_DES_CBC_CRC:
+      rc = shishi_crc (handle, tmp, tmplen, &p);
+      break;
+
     case SHISHI_DES_CBC_MD4:
       rc = shishi_md4 (handle, tmp, tmplen, &p);
       break;
@@ -126,10 +134,11 @@ des_encrypt_checksum (Shishi * handle,
   char *inpad;
   char *pt;
   size_t inpadlen, padzerolen = 0, ptlen, cksumlen;
+  int hlen = (algo == SHISHI_DES_CBC_CRC) ? 4 : 16;
   int res;
 
-  if (inlen % 8)
-    padzerolen = 8 - (inlen % 8);
+  if ((inlen + hlen) % 8)
+    padzerolen = 8 - ((inlen + hlen) % 8);
   inpadlen = inlen + padzerolen;
   inpad = xmalloc (inpadlen);
 
@@ -150,9 +159,8 @@ des_encrypt_checksum (Shishi * handle,
 
   free (inpad);
 
-  res =
-    _shishi_simplified_encrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-				pt, ptlen, out, outlen);
+  res = _shishi_simplified_encrypt (handle, key, 0, iv, ivlen,
+				    ivout, ivoutlen, pt, ptlen, out, outlen);
 
   free (pt);
 
@@ -173,8 +181,9 @@ des_crc_encrypt (Shishi * handle,
 		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_encrypt_checksum (handle, key, keyusage, iv, ivlen, ivout,
-			       ivoutlen, in, inlen, out, outlen,
+  return des_encrypt_checksum (handle, key, keyusage,
+			       shishi_key_value (key), shishi_key_length (key),
+			       ivout, ivoutlen, in, inlen, out, outlen,
 			       SHISHI_DES_CBC_CRC);
 }
 
@@ -231,22 +240,38 @@ des_decrypt_verify (Shishi * handle,
   int res;
   char incoming[16];
   char *computed;
-  size_t hlen = 16;
+  size_t hlen = (algo == SHISHI_DES_CBC_CRC) ? 4 : 16;
 
-  res =
-    _shishi_simplified_decrypt (handle, key, 0, iv, ivlen, ivout, ivoutlen,
-				in, inlen, out, outlen);
+  res = _shishi_simplified_decrypt (handle, key, 0, iv, ivlen,
+				    ivout, ivoutlen, in, inlen, out, outlen);
   if (res != SHISHI_OK)
     {
       shishi_error_printf (handle, "decrypt failed");
       return res;
     }
 
+  if (VERBOSECRYPTONOICE (handle))
+    {
+      puts ("verify decrypted:");
+      _shishi_escapeprint (*out, *outlen);
+      _shishi_hexprint (*out, *outlen);
+    }
+
   memcpy (incoming, *out + 8, hlen);
   memset (*out + 8, 0, hlen);
 
+  if (VERBOSECRYPTONOICE (handle))
+    {
+      puts ("cksum pt:");
+      _shishi_hexprint (*out, *outlen);
+    }
+
   switch (algo)
     {
+    case SHISHI_DES_CBC_CRC:
+      shishi_crc (handle, *out, *outlen, &computed);
+      break;
+
     case SHISHI_DES_CBC_MD4:
       shishi_md4 (handle, *out, *outlen, &computed);
       break;
@@ -261,7 +286,7 @@ des_decrypt_verify (Shishi * handle,
       break;
     }
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       puts ("DES verify:");
       _shishi_hexprint (incoming, hlen);
@@ -291,8 +316,9 @@ des_crc_decrypt (Shishi * handle,
 		 char **ivout, size_t * ivoutlen,
 		 const char *in, size_t inlen, char **out, size_t * outlen)
 {
-  return des_decrypt_verify (handle, key, keyusage, iv, ivlen, ivout,
-			     ivoutlen, in, inlen, out, outlen,
+  return des_decrypt_verify (handle, key, keyusage,
+			     shishi_key_value (key), shishi_key_length (key),
+			     ivout, ivoutlen, in, inlen, out, outlen,
 			     SHISHI_DES_CBC_CRC);
 }
 
@@ -390,7 +416,7 @@ des_key_correction (Shishi * handle, char key[8])
   for (i = 0; i < 16; i++)
     if (memcmp (key, weak_des_keys[i], 8) == 0)
       {
-	if (VERBOSECRYPTO (handle))
+	if (VERBOSECRYPTONOICE (handle))
 	  printf ("\t ;; WEAK KEY (corrected)\n");
 	key[7] ^= 0xF0;
 	break;
@@ -445,6 +471,10 @@ des_string_to_key (Shishi * handle,
       printf ("\t ;; Salt:\n");
       _shishi_escapeprint (salt, saltlen);
       _shishi_hexprint (salt, saltlen);
+    }
+
+  if (VERBOSECRYPTONOICE (handle))
+    {
       printf ("odd = 1;\n");
       printf ("s = string | salt;\n");
       printf ("tempstring = 0; /* 56-bit string */\n");
@@ -463,7 +493,7 @@ des_string_to_key (Shishi * handle,
   memset (s + stringlen + saltlen, 0, n_s - stringlen - saltlen);
   memset (tempkey, 0, sizeof (tempkey));	/* tempkey = NULL; */
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; s = pad(string|salt):\n");
       _shishi_escapeprint (s, n_s);
@@ -472,7 +502,7 @@ des_string_to_key (Shishi * handle,
 
   for (i = 0; i < n_s / 8; i++)
     {
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("for (8byteblock in s) {\n");
 	  printf ("\t ;; loop iteration %d\n", i);
@@ -486,7 +516,7 @@ des_string_to_key (Shishi * handle,
       for (j = 0; j < 8; j++)
 	s[i * 8 + j] = s[i * 8 + j] & ~0x80;
 
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("\t ;; 56bitstring:\n");
 	  _shishi_bin7print (&s[i * 8], 8);
@@ -516,7 +546,7 @@ des_string_to_key (Shishi * handle,
 	      s[i * 8 + j] = temp2;
 	      s[i * 8 + 7 - j] = temp;
 	    }
-	  if (VERBOSECRYPTO (handle))
+	  if (VERBOSECRYPTONOICE (handle))
 	    {
 	      printf ("reverse(56bitstring)\n");
 	      printf ("\t ;; 56bitstring after reverse\n");
@@ -526,7 +556,7 @@ des_string_to_key (Shishi * handle,
 
       odd = !odd;
 
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("odd = ! odd\n");
 	  printf ("tempstring = tempstring XOR 56bitstring;\n");
@@ -536,7 +566,7 @@ des_string_to_key (Shishi * handle,
       for (j = 0; j < 8; j++)
 	tempkey[j] ^= s[i * 8 + j];
 
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("\t ;; tempstring\n");
 	  _shishi_bin7print (tempkey, 8);
@@ -546,7 +576,7 @@ des_string_to_key (Shishi * handle,
   for (j = 0; j < 8; j++)
     tempkey[j] = tempkey[j] << 1;
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("for (8byteblock in s) {\n");
       printf ("}\n");
@@ -562,7 +592,7 @@ des_string_to_key (Shishi * handle,
   if (res != SHISHI_OK)
     return res;
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; tempkey\n");
       _shishi_escapeprint (tempkey, 8);
@@ -640,6 +670,17 @@ des_checksum (Shishi * handle,
   return SHISHI_OK;
 }
 
+static int
+des_crc_checksum (Shishi * handle,
+		  Shishi_key * key,
+		  int keyusage,
+		  int cksumtype,
+		  const char *in, size_t inlen, char **out, size_t * outlen)
+{
+  return des_checksum (handle, key, keyusage, cksumtype,
+		       in, inlen, out, outlen, SHISHI_DES_CBC_CRC);
+}
+
 static int
 des_md4_checksum (Shishi * handle,
 		  Shishi_key * key,
@@ -790,7 +831,7 @@ cipherinfo des_cbc_crc_info = {
   8,
   8,
   8,
-  SHISHI_RSA_MD5_DES,
+  SHISHI_CRC32,
   des_random_to_key,
   des_string_to_key,
   des_crc_encrypt,
@@ -842,6 +883,13 @@ cipherinfo des_cbc_none_info = {
   des_none_decrypt
 };
 
+checksuminfo crc32_info = {
+  SHISHI_CRC32,
+  "crc32",
+  4,
+  des_crc_checksum,
+};
+
 checksuminfo md4_des_info = {
   SHISHI_RSA_MD4_DES,
   "rsa-md4-des",
