@@ -40,7 +40,7 @@ lcm (int a, int b)
 static void
 rot13 (Shishi * handle, char *in, char *out, int len)
 {
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; rot 13 in:\n");
       _shishi_escapeprint (in, len);
@@ -80,7 +80,7 @@ rot13 (Shishi * handle, char *in, char *out, int len)
       out[1] = (in[1] & ~(0xFF & (0xFF << 3))) | (0xFF & (last << 3));
     }
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; rot13 out:\n");
       _shishi_escapeprint (out, len);
@@ -129,10 +129,10 @@ simplified_hmac (Shishi * handle,
 		 const char *in, size_t inlen,
 		 char **outhash, size_t * outhashlen)
 {
-  *outhashlen = 20;
-  return shishi_hmac_sha1 (handle,
-			   shishi_key_value (key), shishi_key_length (key),
-			   in, inlen, outhash);
+  *outhashlen = shishi_checksum_cksumlen
+    (shishi_cipher_defaultcksumtype (shishi_key_type (key)));
+  return shishi_hmac_sha1 (handle, shishi_key_value (key),
+			   shishi_key_length (key), in, inlen, outhash);
 }
 
 static int
@@ -240,41 +240,65 @@ _shishi_simplified_dencrypt (Shishi * handle,
 			     char **out, size_t * outlen, int decryptp)
 {
   int rc;
+  char *pt;
+  size_t ptlen;
+  size_t padzerolen = 0;
 
-  if (outlen)
-    *outlen = inlen;
+  if ((inlen % 8) != 0)
+    while (((inlen + padzerolen) % 8) != 0)
+      padzerolen++;
+
+  ptlen = inlen + padzerolen;
+
+  if (padzerolen)
+    {
+      pt = xmalloc (ptlen);
+      memcpy (pt, in, inlen);
+      memset (pt + inlen, 0, padzerolen);
+    }
+  else
+    pt = (char*) in;
 
   switch (shishi_key_type (key))
     {
     case SHISHI_DES_CBC_CRC:
     case SHISHI_DES_CBC_MD4:
     case SHISHI_DES_CBC_MD5:
+      rc = shishi_des (handle, decryptp, shishi_key_value (key),
+		       iv, ivout, pt, ptlen, out);
       if (ivoutlen)
 	*ivoutlen = 8;
-      rc = shishi_des (handle, decryptp, shishi_key_value (key),
-		       iv, ivout, in, inlen, out);
+      if (outlen)
+	*outlen = ptlen;
       break;
 
     case SHISHI_DES3_CBC_HMAC_SHA1_KD:
+      rc = shishi_3des (handle, decryptp, shishi_key_value (key),
+			iv, ivout, pt, inlen + padzerolen, out);
       if (ivoutlen)
 	*ivoutlen = 8;
-      rc = shishi_3des (handle, decryptp, shishi_key_value (key),
-			iv, ivout, in, inlen, out);
+      if (outlen)
+	*outlen = ptlen;
       break;
 
     case SHISHI_AES128_CTS_HMAC_SHA1_96:
     case SHISHI_AES256_CTS_HMAC_SHA1_96:
-      if (ivoutlen)
-	*ivoutlen = 16;
       rc = shishi_aes_cts (handle, decryptp,
 			   shishi_key_value (key), shishi_key_length (key),
 			   iv, ivout, in, inlen, out);
+      if (ivoutlen)
+	*ivoutlen = 16;
+      if (outlen)
+	*outlen = inlen;
       break;
 
     default:
       rc = SHISHI_CRYPTO_ERROR;
     }
 
+  if (padzerolen)
+    free (pt);
+
   return rc;
 }
 
@@ -307,7 +331,7 @@ _shishi_simplified_encrypt (Shishi * handle,
       ptlen = inlen + blen + padzerolen;
       pt = xmalloc (ptlen);
 
-      res = shishi_randomize (handle, pt, blen);
+      res = shishi_randomize (handle, 0, pt, blen);
       if (res != SHISHI_OK)
 	goto done;
 
@@ -380,7 +404,8 @@ _shishi_simplified_decrypt (Shishi * handle,
     {
       Shishi_key *privacykey = NULL, *integritykey = NULL;
       int blen = shishi_cipher_blocksize (shishi_key_type (key));
-      size_t hlen = 20;		/* XXX only works for SHA-1 */
+      size_t hlen = shishi_checksum_cksumlen
+	(shishi_cipher_defaultcksumtype (shishi_key_type (key)));
 
       res = _shishi_simplified_derivekey (handle, key, keyusage,
 					  SHISHI_DERIVEKEYMODE_PRIVACY,
@@ -635,6 +660,22 @@ shishi_cipher_defaultcksumtype (int32_t type)
   return -1;
 }
 
+static struct {
+  char *name;
+  int type;
+} cipher_aliases[] = {
+  { "des-crc", SHISHI_DES_CBC_CRC },
+  { "des-md4", SHISHI_DES_CBC_MD4 },
+  { "des-md5", SHISHI_DES_CBC_MD5 },
+  { "des", SHISHI_DES_CBC_MD5 },
+  { "des3", SHISHI_DES3_CBC_HMAC_SHA1_KD },
+  { "3des", SHISHI_DES3_CBC_HMAC_SHA1_KD },
+  { "aes128", SHISHI_AES128_CTS_HMAC_SHA1_96 },
+  { "aes256", SHISHI_AES256_CTS_HMAC_SHA1_96 },
+  { "aes", SHISHI_AES256_CTS_HMAC_SHA1_96 },
+  { "arcfour", SHISHI_ARCFOUR_HMAC }
+};
+
 /**
  * shishi_cipher_parse:
  * @cipher: name of encryption type, e.g. "des3-cbc-sha1-kd".
@@ -656,6 +697,10 @@ shishi_cipher_parse (const char *cipher)
     if (strcasecmp (cipher, ciphers[i]->name) == 0)
       return ciphers[i]->type;
 
+  for (i = 0; i < sizeof (cipher_aliases) / sizeof (cipher_aliases[0]); i++)
+    if (strcasecmp (cipher, cipher_aliases[i].name) == 0)
+      return cipher_aliases[i].type;
+
   return -1;
 }
 
@@ -708,6 +753,9 @@ _shishi_cipher_decrypt (int32_t type)
 }
 
 static checksuminfo *checksums[] = {
+#if WITH_DES
+  &crc32_info,
+#endif
 #if WITH_MD
   &md4_info,
 #endif
@@ -1449,6 +1497,12 @@ shishi_decrypt_ivupdate_etype (Shishi * handle,
       printf ("\t ;; in (%d):\n", inlen);
       _shishi_escapeprint (in, inlen);
       _shishi_hexprint (in, inlen);
+      if (iv)
+	{
+	  printf ("\t ;; iv (%d):\n", ivlen);
+	  _shishi_escapeprint (iv, ivlen);
+	  _shishi_hexprint (iv, ivlen);
+	}
     }
 
   decrypt = _shishi_cipher_decrypt (etype);
@@ -1735,7 +1789,7 @@ shishi_n_fold (Shishi * handle,
 
   lcmmn = lcm (m, n);
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("%d-fold (string)\n", n * 8);
       printf ("\t ;; string length %d bytes %d bits\n", m, m * 8);
@@ -1750,7 +1804,7 @@ shishi_n_fold (Shishi * handle,
   /* Replicate the input th the LCMMN length */
   for (i = 0; i < (lcmmn / m); i++)
     {
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("\t ;; %d-th replication\n", i + 1);
 	  printf ("string = rot13(string)\n");
@@ -1762,7 +1816,7 @@ shishi_n_fold (Shishi * handle,
 
   memset (out, 0, n);		/* just in case */
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; replicated string (length %d):\n", lcmmn);
       _shishi_hexprint (buf, lcmmn);
@@ -1777,7 +1831,7 @@ shishi_n_fold (Shishi * handle,
 
   for (i = 0; i < (lcmmn / n); i++)
     {
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("\t ;; %d-th one's complement addition sum\n", i + 1);
 	  printf ("\t ;; sum:\n");
@@ -1791,7 +1845,7 @@ shishi_n_fold (Shishi * handle,
 
       ocadd (out, (char *) &buf[i * n], out, n);
 
-      if (VERBOSECRYPTO (handle))
+      if (VERBOSECRYPTONOICE (handle))
 	{
 	  printf ("\t ;; sum:\n");
 	  _shishi_hexprint (out, n);
@@ -1799,7 +1853,7 @@ shishi_n_fold (Shishi * handle,
 	}
     }
 
-  if (VERBOSECRYPTO (handle))
+  if (VERBOSECRYPTONOICE (handle))
     {
       printf ("\t ;; nfold\n");
       _shishi_hexprint (out, n);
