@@ -63,9 +63,51 @@ _shishi_crypto_init (Shishi * handle)
 }
 
 int
-shishi_randomize (Shishi * handle, char *data, size_t datalen)
+shishi_randomize (Shishi * handle, int strong, char *data, size_t datalen)
 {
-  gcry_randomize (data, datalen, GCRY_STRONG_RANDOM);
+  if (strong)
+    gcry_randomize (data, datalen, GCRY_VERY_STRONG_RANDOM);
+  else
+    gcry_randomize (data, datalen, GCRY_STRONG_RANDOM);
+  return SHISHI_OK;
+}
+
+int
+shishi_crc (Shishi * handle,
+	    const char *in, size_t inlen,
+	    char *out[4])
+{
+  gcry_md_hd_t hd;
+  gpg_error_t err;
+  char T[4];
+  char *p;
+
+  err = gcry_md_open (&hd, GCRY_MD_CRC32_RFC1510, 0);
+  if (err != GPG_ERR_NO_ERROR)
+    {
+      shishi_error_printf (handle, "Libgcrypt cannot open CRC handle");
+      shishi_error_set (handle, gpg_strerror (err));
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+
+  gcry_md_write (hd, in, inlen);
+
+  p = gcry_md_read (hd, GCRY_MD_CRC32_RFC1510);
+  if (p == NULL)
+    {
+      shishi_error_printf (handle, "Libgcrypt failed to compute hash");
+      return SHISHI_CRYPTO_INTERNAL_ERROR;
+    }
+
+  *out = xmalloc (4);
+  /* XXX */
+  (*out)[0] = p[3];
+  (*out)[1] = p[2];
+  (*out)[2] = p[1];
+  (*out)[3] = p[0];
+
+  gcry_md_close (hd);
+
   return SHISHI_OK;
 }
 
@@ -182,7 +224,7 @@ int
 shishi_hmac_sha1 (Shishi * handle,
 		  const char *key, size_t keylen,
 		  const char *in, size_t inlen,
-		  char *outhash[16])
+		  char *outhash[20])
 {
   gcry_md_hd_t mdh;
   size_t hlen = gcry_md_get_algo_dlen (GCRY_MD_SHA1);
@@ -274,7 +316,8 @@ shishi_des_cbc_mac (Shishi * handle,
 }
 
 static int
-libgcrypt_dencrypt (Shishi * handle, int algo, int flags, int decryptp,
+libgcrypt_dencrypt (Shishi * handle, int algo, int flags, int mode,
+		    int decryptp,
 		    const char *key, size_t keylen,
 		    const char *iv,
 		    char **ivout,
@@ -284,7 +327,6 @@ libgcrypt_dencrypt (Shishi * handle, int algo, int flags, int decryptp,
   size_t ivlen = gcry_cipher_get_algo_blklen (algo);
   gcry_cipher_hd_t ch;
   gpg_error_t err;
-  int mode = GCRY_CIPHER_MODE_CBC;
 
   err = gcry_cipher_open (&ch, algo, mode, flags);
   if (err != GPG_ERR_NO_ERROR)
@@ -347,11 +389,14 @@ libgcrypt_dencrypt (Shishi * handle, int algo, int flags, int decryptp,
 int
 shishi_arcfour (Shishi * handle, int decryptp,
 		const char *key, size_t keylen,
+		const char iv[258], char *ivout[258],
 		const char *in, size_t inlen,
 		char **out)
 {
+  /* XXX Support iv/ivout. */
   return libgcrypt_dencrypt (handle, GCRY_CIPHER_ARCFOUR, 0,
-			     decryptp, key, keylen, NULL, NULL, in, inlen, out);
+			     GCRY_CIPHER_MODE_STREAM, decryptp,
+			     key, keylen, NULL, NULL, in, inlen, out);
 }
 
 int
@@ -362,7 +407,7 @@ shishi_des (Shishi * handle, int decryptp,
 	    const char *in, size_t inlen,
 	    char **out)
 {
-  return libgcrypt_dencrypt (handle, GCRY_CIPHER_DES, 0,
+  return libgcrypt_dencrypt (handle, GCRY_CIPHER_DES, 0, GCRY_CIPHER_MODE_CBC,
 			     decryptp, key, 8, iv, ivout, in, inlen, out);
 }
 
@@ -374,7 +419,7 @@ shishi_3des (Shishi * handle, int decryptp,
 	     const char *in, size_t inlen,
 	     char **out)
 {
-  return libgcrypt_dencrypt (handle, GCRY_CIPHER_3DES, 0,
+  return libgcrypt_dencrypt (handle, GCRY_CIPHER_3DES, 0, GCRY_CIPHER_MODE_CBC,
 			     decryptp, key, 24, iv, ivout, in, inlen, out);
 }
 
@@ -387,5 +432,6 @@ shishi_aes_cts (Shishi * handle, int decryptp,
 		char **out)
 {
   return libgcrypt_dencrypt (handle, GCRY_CIPHER_AES, GCRY_CIPHER_CBC_CTS,
-			     decryptp, key, keylen, iv, ivout, in, inlen, out);
+			     GCRY_CIPHER_MODE_CBC, decryptp,
+			     key, keylen, iv, ivout, in, inlen, out);
 }
