@@ -27,7 +27,7 @@
 #include <time.h>		/* time_t */
 #include <shishi-int.h>		/* uint32_t */
 
-#define SHISHI_VERSION "0.0.6"
+#define SHISHI_VERSION "0.0.7"
 
 /* Error codes */
 typedef enum
@@ -1260,7 +1260,10 @@ extern int shishi_time (Shishi * handle, Shishi_asn1 node,
 			const char *field, char **time);
 
 /* nettle.c, libgcrypt.c, ... */
-extern int shishi_randomize (Shishi * handle, char *data, size_t datalen);
+extern int shishi_randomize (Shishi * handle, int strong,
+			     char *data, size_t datalen);
+extern int shishi_crc (Shishi * handle, const char *in, size_t inlen,
+		       char *out[4]);
 extern int shishi_md4 (Shishi * handle, const char *in, size_t inlen,
 		       char *out[16]);
 extern int shishi_md5 (Shishi * handle, const char *in, size_t inlen,
@@ -1272,6 +1275,10 @@ extern int shishi_hmac_sha1 (Shishi * handle, const char *key, size_t keylen,
 extern int shishi_des_cbc_mac (Shishi * handle, const char key[8],
 			       const char iv[8], const char *in, size_t inlen,
 			       char *out[8]);
+extern int shishi_arcfour (Shishi * handle, int decryptp,
+			   const char *key, size_t keylen,
+			   const char iv[258], char *ivout[258],
+			   const char *in, size_t inlen, char **out);
 extern int shishi_des (Shishi * handle, int decryptp, const char key[8],
 		       const char iv[8], char *ivout[8],
 		       const char *in, size_t inlen, char **out);
