@@ -173,7 +173,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 
     case 'v':
-      arguments->verbose = 1;
+      arguments->verbose++;
       break;
 
     case 'c':
@@ -214,6 +214,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	  if (arguments->listenspec == NULL)
 	    argp_error (state, "Fatal memory allocation error");
 	  ls = &arguments->listenspec[arguments->nlistenspec - 1];
+	  memset (ls, 0, sizeof(*ls));
 	  ls->str = strdup (val);
 	  ls->bufpos = 0;
 	  sin = (struct sockaddr_in *) &ls->addr;
@@ -482,7 +483,7 @@ asreq1 (Shishi * handle, struct arguments *arg, Shishi_as * as)
   return SHISHI_OK;
 }
 
-static void
+static int
 asreq (Shishi * handle, struct arguments *arg,
        Shishi_asn1 kdcreq, char **out, size_t *outlen)
 {
@@ -497,7 +498,7 @@ asreq (Shishi * handle, struct arguments *arg,
       /* XXX hard coded KRB-ERROR? */
       *out = strdup ("foo");
       *outlen = strlen (*out);
-      return;
+      return rc;
     }
 
   shishi_as_req_set (as, kdcreq);
@@ -520,11 +521,12 @@ asreq (Shishi * handle, struct arguments *arg,
       /* XXX hard coded KRB-ERROR? */
       *out = strdup ("aaaaaa");
       *outlen = strlen (*out);
-      return;
+      return rc;
     }
 
-  return;
+  return SHISHI_OK;
 }
+
 static int
 tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
 {
@@ -535,7 +537,7 @@ tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
   int buflen;
   int err;
   char *username, *servername, *realm;
-  int32_t etype;
+  int32_t etype, keyusage;
   int i;
 
   tkt = shishi_tgs_tkt (tgs);
@@ -634,11 +636,17 @@ tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
     return err;
 
   if (err == SHISHI_OK)
-    keytouse = subkey;
+    {
+      keyusage = SHISHI_KEYUSAGE_ENCTGSREPPART_AUTHENTICATOR_KEY;
+      keytouse = subkey;
+    }
   else
-    keytouse = sessiontktkey;
+    {
+      keyusage = SHISHI_KEYUSAGE_ENCTGSREPPART_SESSION_KEY;
+      keytouse = sessiontktkey;
+    }
 
-  err = shishi_tgs_rep_build (tgs, keytouse);
+  err = shishi_tgs_rep_build (tgs, keyusage, keytouse);
   if (err)
     return err;
 
@@ -675,7 +683,7 @@ tgsreq1 (Shishi * handle, struct arguments *arg, Shishi_tgs * tgs)
   return SHISHI_OK;
 }
 
-static void
+static int
 tgsreq (Shishi * handle, struct arguments *arg,
 	Shishi_asn1 kdcreq, char **out, size_t *outlen)
 {
@@ -690,7 +698,7 @@ tgsreq (Shishi * handle, struct arguments *arg,
       /* XXX hard coded KRB-ERROR? */
       *out = strdup ("foo");
       *outlen = strlen (*out);
-      return;
+      return rc;
     }
 
   shishi_tgs_req_set (tgs, kdcreq);
@@ -713,27 +721,17 @@ tgsreq (Shishi * handle, struct arguments *arg,
       /* XXX hard coded KRB-ERROR? */
       *out = strdup ("aaaaaa");
       *outlen = strlen (*out);
-      return;
+      return rc;
     }
 
-  return;
-}
-
-static Shishi_msgtype
-get_msgtype (Shishi * handle, char *in, size_t inlen)
-{
-  if (inlen > 1 && *in >= 0x60 && (unsigned char) *in <= 0x7F)
-    return *in - 0x60;
-  else
-    return 0;
+  return SHISHI_OK;
 }
 
 static int
 process_1 (Shishi * handle, struct arguments *arg,
 	   char *in, size_t inlen, char **out, size_t * outlen)
 {
-  Shishi_asn1 kdcreq;
-  Shishi_msgtype msgtype;
+  Shishi_asn1 node;
   Shishi_asn1 krberr;
   int rc;
 
@@ -741,57 +739,37 @@ process_1 (Shishi * handle, struct arguments *arg,
   if (!krberr)
     return SHISHI_MALLOC_ERROR;
 
-  fprintf (stderr, "Processing %d bytes...\n", inlen);
+  node = shishi_der2asn1 (handle, in, inlen);
 
-  msgtype = get_msgtype (handle, in, inlen);
+  fprintf (stderr, "ASN.1 msg-type %d (0x%x)...\n",
+	   shishi_asn1_msgtype (handle, node),
+	   shishi_asn1_msgtype (handle, node));
 
-  fprintf (stderr, "ASN.1 msg-type %d (0x%x)...\n", msgtype, msgtype);
-
-  switch (msgtype)
+  switch (shishi_asn1_msgtype (handle, node))
     {
     case SHISHI_MSGTYPE_AS_REQ:
-      kdcreq = shishi_der2asn1_asreq (handle, in, inlen);
-      if (kdcreq)
-	{
-	  asreq (handle, arg, kdcreq, out, outlen);
-	  return SHISHI_OK;
-	}
-      else
-	{
-	  rc = shishi_krberror_set_etext (handle, krberr,
-					  "Cannot parse AS-REQ");
-	  if (rc != SHISHI_OK)
-	    return rc;
-	}
+      rc = asreq (handle, arg, node, out, outlen);
       break;
 
     case SHISHI_MSGTYPE_TGS_REQ:
-      kdcreq = shishi_der2asn1_tgsreq (handle, in, inlen);
-      if (kdcreq)
-	{
-	  tgsreq (handle, arg, kdcreq, out, outlen);
-	  return SHISHI_OK;
-	}
-      else
-	{
-	  rc = shishi_krberror_set_etext (handle, krberr,
-					  "Cannot parse TGS-REQ");
-	  if (rc != SHISHI_OK)
-	    return rc;
-	}
+      rc = tgsreq (handle, arg, node, out, outlen);
       break;
 
     default:
-      rc = shishi_krberror_set_etext (handle, krberr,
-				      "Unsupported message type");
-      if (rc != SHISHI_OK)
-	return rc;
+      rc = !SHISHI_OK;
       break;
     }
 
-  rc = shishi_krberror_der (handle, krberr, out, outlen);
   if (rc != SHISHI_OK)
-    return rc;
+    {
+      rc = shishi_krberror_set_etext (handle, krberr, "General error");
+      if (rc != SHISHI_OK)
+	return rc;
+
+      rc = shishi_krberror_der (handle, krberr, out, outlen);
+      if (rc != SHISHI_OK)
+	return rc;
+    }
 
   return SHISHI_OK;
 }
@@ -910,7 +888,6 @@ kdc_loop (Shishi * handle, struct arguments *arg)
 
   while (!quit)
     {
-      sleep (1);
       do
 	{
 	  FD_ZERO (&readfds);
@@ -981,7 +958,7 @@ kdc_loop (Shishi * handle, struct arguments *arg)
 		    close (ls->sockfd);
 		    ls->sockfd = 0;
 		  }
-		else
+		else if (read_bytes > 0)
 		  {
 		    ls->bufpos += read_bytes;
 		    ls->buf[ls->bufpos] = '\0';
@@ -1154,6 +1131,18 @@ init (struct arguments *arg)
       return 1;
     }
 
+  if (arg->verbose > 1)
+    shishi_cfg (handle, "verbose");
+
+  if (arg->verbose > 2)
+    shishi_cfg (handle, "verbose-noice");
+
+  if (arg->verbose > 3)
+    shishi_cfg (handle, "verbose-asn1");
+
+  if (arg->verbose > 4)
+    shishi_cfg (handle, "verbose-crypto");
+
   rc = setup (handle, arg);
 
   shishi_done (handle);
